#ifndef _IS_THIS_DEBUGGING_HEADER_DEFINED_
#define _IS_THIS_DEBUGGING_HEADER_DEFINED_

/* comment/uncomment the following line to control 
   verbose output debugging and file logging 
 */
#define AI_DEBUG

/* help find memory leaks */
#define NO_LEAKS 0    /* compared to return value of debug_leaks() */
#define ADD_MALLOC 0  /* values passed to debug_leaks() */
#define SUB_FREE 1    
#define GET_DMA_COUNT 2
int debug_leaks(char); /* counts mallocs and frees */


/*---------------------------------*/
/* slower code but this helps tracks down bugs */
#ifdef AI_DEBUG
   /* error return values */
   #define ERROR_FREE 10
   
   /* macros to help with runtime debugging */
   #define DBG(x,y) {printf("\n\n%s (%s, line %d)\n\n",x, __FILE__, __LINE__); return(y);}
   
   #define DEBUG_MALLOC(cast,ret,algn,size,fail_on) if (((cast*)ret = (cast*)memalign(algn,size)) == fail_on){DBG("failed to alloc dynamic memory",NULL)} else debug_leaks(ADD_MALLOC); 
 
   #define DEBUG_FREE(x) if (x) { free(x); debug_leaks(SUB_FREE);} else DBG("attempting to free unallocated memory",ERROR_FREE)

   #define DBG_WARN(x) {printf("\n\n%s (%s: line %d)\n\n",x,__FILE__, __LINE__);}
#endif


/*---------------------------------*/
/* debugging?  No! Not for you, my friend... */
#ifndef AI_DEBUG
   /* compiler optimizations will remove return value checks */
   #define DBG(x,y) {}
   #define DEBUG_MALLOC(cast,ret,algn,size,fail_on) ret = (cast*)memalign(algn,size);
   #define DEBUG_FREE(x) free(x);
   #define DBG_WARN(x) {}
#endif

#endif
