#ifndef _IS_THIS_NEURAL_MLFN_HEADER_DEFINED_
#define _IS_THIS_NEURAL_MLFN_HEADER_DEFINED_

/* All the beauty that is the Multi-Layer Feedforward Network 
   (now you know why everything is named with MLFN)
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h> /* because memalign() isnt in stdlib.h */
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "mlfn_approx.h"
#include "micro.vh"
#include "aidoru_types.h"
#include "debugging.h"

/* -------------------defines-------------------*/

/* no layer may have more than 16 neurons (4*4) 
   too much code depends on this so it cant be
   changed at the time 
*/
#define MAX_NEURON_VECS 4
#define MAX_NEURONS MAX_NEURON_VECS * VECTOR_SIZE
/* allocate a new net when loading??? */
#define CREATE 1
#define NO_CREATE 0
/* end of file marker for a MLFN file */
#define END_SYNAPSE_FILE 0xAA

/*---------------------macros-------------------*/

/* load neurons into vf10, vf11, vf12, vf13 */
#define LOAD_NEURONS(x) asm __volatile__ ("lqc2 vf10, 0x0(%0);                                                       "::"r"(x));

/* load synapses into vf14, vf15, vf16, vf17 */
#define LOAD_SYNAPSES(vec,n) asm __volatile__ ("lqc2 vf14, 0(%0);                                                          lqc2 vf15, 16(%0);                                                         lqc2 vf16, 32(%0);                                                         lqc2 vf17, 48(%0);                                                         lw $10,0x0(%1);                                                            ctc2 $10,$vi01;                                                            "::"r"(vec),"r"(&n)); 
                                             

/*---------------------types--------------------*/

/* a struct to hold neuron layers... */
typedef struct
{ 
  /* neurons */
  ps2_vu0_fvector *neuron;
  /* synapse[neuron_num][vector num][synapse] */
  ps2_vu0_fvector **synapse; 
  int vects;
}mlfn_layer;

/* struct to represent a multi-layer feedforward net */
typedef struct
{
  unsigned char num_layers;    /* num of input/hidden/output layers */ 
  mlfn_layer *layer;           /* actual layers */
  char *num_neurons;           /* num neurons per layer */
}mlfn_net;

/*---------------------prototypes---------------------*/

/* create and init a mlfn neural network */
mlfn_net *create_mlfn_network(const char,const char*);

/* returned malloc-ed memory */
char destroy_mlfn_net(mlfn_net*);

/* calculate the output for a given input */
int mlfn_fwd_propagate(mlfn_net*);

/* calculate the numbers of vectors needed to represent neurons */
int vectors_needed(const char);

/* load and save a previously trained net */
mlfn_net *load_mlfn(char *, mlfn_net*, char);
int save_mlfn(char *, mlfn_net*);

#endif
